{

=pod

=head1 NAME

WWW::Dictionarydotcom

=head1 SUMMARY

 use WWW::Dictionarydotcom;
 use Data::Dumper; 

 print &Dumper(WWW::Dictionarydotcom->new()->wotd());'

 # Prints :
 $VAR1 = {
          'etymology' => ' Surreptitious comes from Latin surrepticius , "stolen, secret, surreptitious," from surripere , "to take away secretly; to steal," from sub- , "under" + rapere , "to seize, to snatch."',
          'definition' => ' 1. Done, made, or gotten by stealth. 2. Acting with or marked by stealth.',
          'permalink' => 'http://www.dictionary.com/wordoftheday/archive/2002/01/07.html',
          'word' => 'surreptitious',
          'usage' => [
                       ' The monitoring is not surreptitious ; on the contrary, the defendant and his or her attorney are required to be given notice of the government\'s listening activities. --John Ashcroft, "National Security; Prevention of Acts of Violence and Terrorism," Federal Register 66, no. 211, October 26, 2001',
                       ' While men\'s appetites are driven by availability, women\'s are often driven by cravings. A dab of chocolate here, a pinch of sugar there, and some surreptitious midnight Dairy Queen runs lurk behind a woman\'s oh-so-virtuous bran breakfast, salad lunch, and grilled fish dinner. --Wendy Hubbert, "The skinny on male/female dieting," Redbook , October 1, 2001',
                       ' Now she made a surreptitious glance toward the doorway into the hall. --Naeem Murr, The Boy'
                     ]
        };

=head1 DESCRIPTION

This packages exports a single class method names I<wotd> which returns a hash reference containing information about the dictionary.com word of the day.

=cut

package WWW::Dictionarydotcom;
use strict;

$WWW::Dictionarydotcom::VERSION   = 0.1;
@WWW::Dictionarydotcom::ISA       = qw ( Exporter );
@WWW::Dictionarydotcom::EXPORT    = qw ();
@WWW::Dictionarydotcom::EXPORT_OK = qw ();

use Carp;
use Data::Dumper;
use Exporter;

use LWP::UserAgent;
use HTTP::Request;
use HTML::Parser;
use XML::RSS;

use constant WOTD_ARCHIVE => "http://www.dictionary.com/wordoftheday/archive";
use constant WOTD_RSS     => "http://www.dictionary.com/wordoftheday/wotd.rss";

my %_selves = ();

# These all need to be stored 
# in $selves{$self->{'_key'}}

my %wotd      = ();
my $okay      = 0;
my $eod       = 0;
my $isquote   = 0;
my $quote     = 0;
my $etymology = 0;

=pod

=head1 CONSTRUCTOR METHODS

=cut

=pod

=head2 $pkg = WWW::Dictionarydotcom->new()

=cut

sub new {
    my $pkg = shift;

    my $self = {};
    bless $self,$pkg;

    if (! $self->init(@_)) {
        return undef;
    }

    return $self;
}

sub init {
    my $self = shift;
    my $args = { @_ };

    my $key = undef;

    while ((! $key) || (exists $_selves{ $key })) {
        $key = rand;
    }

    $_selves{ $key }    = {};
    $self->{ "_key" } = $key;

    return 1;
}

=pod

=head2 $pkg->wotd($date)

I<$date> should be formatted as "YYYY-MM-DD"

If no date is specified, the method parses the word of the day as defined by the current dictionary.com word of the day RSS file.

=cut

sub wotd {
    my $self = shift;
    my $date = shift;

    %wotd      = ();
    $okay      = 0;
    $eod       = 0;
    $isquote   = 0;
    $quote     = 0;
    $etymology = 0;
    
    my $uri = undef;

    if (! $date) {
	$uri  = &_parserss(&_fetch(WOTD_RSS));
    } else {
	$date =~ s/-/\//g;
	$uri = WOTD_ARCHIVE."/$date.html";
    }

    my $html = &_fetch($uri);

    my $parser = HTML::Parser->new(
                                   start_h   => [\&_start,   "self, tagname, attr, attrseq, text"],
                                   end_h     => [\&_end,     "self, tagname"],
                                   text_h    => [\&_text,    "self, text, is_cdata"],
				   comment_h => [\&_comment, "self, text"],
                                   );

    $parser->parse($$html);

    if (! $wotd{ word }) {
	carp "Unable to parse wotd";
	return {};
    }

    $wotd{ permalink } = $uri;
    return \%wotd;
}

=pod

=head1 PRIVATE METHODS

=cut

sub _fetch {
    my $uri = shift;

    my $ua  = new LWP::UserAgent;
    my $res = $ua->request(HTTP::Request->new(GET=>$uri));
	
    if (! $res->is_success()) {
	carp join(":",$res->code(),$res->message()),"\n";
	return undef;
    }

    return \ $res->content();
}

sub _parserss {
    my $xml = shift;

    my $rss = XML::RSS->new();
    $rss->parse($$xml);
    
    return $rss->{'items'}[0]->{'link'};
}

#

sub _start {
    my $parser = shift;
    my $tag    = shift;
    my $attrs  = shift;

    return if (! $okay);

    if (($eod) && ($tag eq "blockquote")) {
	$isquote = 1;
    }

    if (($eod) && ($tag eq "hr")) {
	$etymology = 1;
    }
}

sub _end {
    my $parser = shift;
    my $tag = shift;

    return if (! $okay);

    if (($eod) && ($tag eq "blockquote")) {
	$isquote = 0;
    }

    if (($isquote) && ($tag eq "p")) {
	$quote++;
    }
}

sub _comment {
    my $parser = shift;
    my $tokens = shift;
    
    if (ref($tokens) ne "ARRAY") {
	$tokens = [ $tokens ];
    }

    for (@$tokens) {
	&_token($_);
    }
}

sub _token {
    my $tok  = shift;

    $tok =~ s/^(\s+)//;
    $tok =~ s/(\s+)$//;

    if ($tok eq "<!-- Begin content -->") {
	$okay = 1;
    }

    if ($tok eq "<!--End content-->") {
	$okay = 0;
    }

    if ($tok =~ /^(<!--\sWOTD=")(.*)?("\s-->)$/) {
	$wotd{ word } = $2;
    }

    if ($tok eq "<!-- SECBR -->") {
	$eod = 1;
    }

    return unless ($wotd{'word'});
}

sub _text {
    my $parser = shift;
    my $text   = shift;

    return if (! $okay);

    $text =~ s/^(\s+)//;
    $text =~ s/(\s+)$//;
    return unless ($text);

    return unless ($wotd{'word'});

    if (($wotd{ word }) && (! $eod)) {
	$wotd{ definition } .= " $text";
    }

    if ($isquote) {
	$wotd{ usage }->[ $quote ] .= " $text";
    }

    if ($etymology) {
	$wotd{ etymology } .= " $text";
    }
}

=pod

=head1 TO DO

=over 4 

=item

Finish encapsulation

=back

=head1 VERSION

0.1

=head1 DATE

$Date: 2002/01/07 14:02:56 $

=head1 AUTHOR

Aaron Straup Cope

=head1 CHANGES

$Log: Dictionarydotcom.pm,v $
Revision 1.4  2002/01/07 14:02:56  asc
Updated POD

Revision 1.3  2002/01/07 14:01:54  asc
Updated POD; fixed typos (thanks jessamyn)

Revision 1.2  2002/01/07 05:37:31  asc
Formatting

Revision 1.1  2002/01/07 05:33:31  asc
Initial commit


=head1 LICENSE

Copyright (c) 2002 Aaron Straup Cope.

This is free software, you may use it and distribute it under the same terms as Perl itself.

=cut

return 1;

}
